@@ -1,10 +1,10 @@
 /*
- * Copyright 2005-2006 Gentoo Foundation
+ * Copyright 2005-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.70 2007/05/23 03:22:31 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.75 2007/06/03 19:17:39 solar Exp $
  *
- * Copyright 2005-2006 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2006 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #ifdef APPLET_qmerge
@@ -53,7 +53,7 @@ static const char *qmerge_opts_help[] = {
 	COMMON_OPTS_HELP
 };
 
-static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.70 2007/05/23 03:22:31 solar Exp $";
+static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.75 2007/06/03 19:17:39 solar Exp $";
 #define qmerge_usage(ret) usage(ret, QMERGE_FLAGS, qmerge_long_opts, qmerge_opts_help, lookup_applet_idx("qmerge"))
 
 char search_pkgs = 0;
@@ -195,8 +195,6 @@ void fetch(const char *destdir, const char *src)
 
 void qmerge_initialize(const char *Packages)
 {
-	char *pbuf = NULL;
-
 	if (strlen(BUSYBOX))
 		if (access(BUSYBOX, X_OK) != 0)
 			err(BUSYBOX " must be installed");
@@ -221,27 +219,16 @@ void qmerge_initialize(const char *Packages)
 	if (chdir("portage") != 0)
 		errf("!!! chdir(%s/portage) %s", port_tmpdir, strerror(errno));
 
-	xasprintf(&pbuf, "%s.bz2", Packages);
-	if (force_download && force_download != 2) {
+	if (force_download && force_download != 2)
 		unlink(Packages);
-		unlink(pbuf);
-	}
 
 	if ((access(Packages, R_OK) != 0) && (force_download != 2)) {
 			char *tbuf = NULL;
 			xasprintf(&tbuf, "%s/portage/", port_tmpdir);
-			fetch(tbuf, pbuf);
-			if ((access(pbuf, R_OK)) == 0) {
-				char *buf = NULL;
-				xasprintf(&buf, "bunzip2 %s", pbuf);
-				system(buf);
-				free(buf);
-			}
 		        if ((access(Packages, R_OK) != 0))
 				fetch(tbuf, Packages);
 			free(tbuf);
 	}
-	free(pbuf);
 }
 
 char *best_version(const char *CATEGORY, const char *PN)
@@ -964,11 +951,12 @@ int pkg_verify_checksums(char *fname, struct pkg_t *pkg, depend_atom *atom, int
 		return ret;
 
 	if (pkg->MD5[0]) {
-		if ((hash = (char*) hash_file(fname, HASH_MD5)) == NULL)
-			errf("hash is NULL for %s", fname);
+		if ((hash = (char*) hash_file(fname, HASH_MD5)) == NULL) {
+			errf("hash is NULL for %s %s", fname, get_current_dir_name());
+		}
 		if (strcmp(hash, pkg->MD5) == 0) {
 			if (display)
-				qprintf("MD5:  [%sOK%s] %s %s/%s\n", GREEN, NORM, hash, atom->CATEGORY, pkg->PF);
+				printf("MD5:  [%sOK%s] %s %s/%s\n", GREEN, NORM, hash, atom->CATEGORY, pkg->PF);
 		} else {
 			if (display)
 				warn("MD5:  [%sER%s] (%s) != (%s) %s/%s", RED, NORM, hash, pkg->MD5, atom->CATEGORY, pkg->PF);
@@ -1000,6 +988,7 @@ int pkg_verify_checksums(char *fname, struct pkg_t *pkg, depend_atom *atom, int
 void pkg_fetch(int argc, char **argv, struct pkg_t *pkg)
 {
 	depend_atom *atom;
+	char savecwd[_POSIX_PATH_MAX];
 	char buf[255], str[255];
 	int i;
 
@@ -1062,6 +1051,7 @@ void pkg_fetch(int argc, char **argv, struct pkg_t *pkg)
 			snprintf(buf, sizeof(buf), "%s.tbz2", pkg->PF);
 			fetch(str, buf);
 		}
+
 		/* verify the pkg exists now. unlink if zero bytes */
 		snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2", pkgdir, atom->CATEGORY, pkg->PF);
 		unlink_empty(buf);
@@ -1071,7 +1061,17 @@ void pkg_fetch(int argc, char **argv, struct pkg_t *pkg)
 			fflush(stderr);
 			continue;
 		}
+		getcwd(savecwd, sizeof(savecwd));
+		assert(chdir(pkgdir) == 0);
+		if (chdir("All/") == 0) {
+			snprintf(buf, sizeof(buf), "%s.tbz2", pkg->PF);
+			snprintf(str, sizeof(str), "../%s/%s.tbz2", atom->CATEGORY, pkg->PF);
+			unlink(buf);
+			symlink(str, buf);
+		}
+		chdir(savecwd);
 
+		snprintf(buf, sizeof(buf), "%s/%s/%s.tbz2", pkgdir, atom->CATEGORY, pkg->PF);
 		if ((pkg_verify_checksums(buf, pkg, atom, qmerge_strict, 1)) == 0) {
 			pkg_merge(0, atom, pkg);
 			continue;
